#!/bin/sh

CLAN=/home/daniel/polyhedral_perforation/clan-0.8.0/clan
PERF=./build/perf
CLOOG=/home/daniel/polyhedral_perforation/pluto/cloog-isl/cloog

usage() {
    printf -- "%s: <OPTIONS> <INPUT>\n" "$0"
    printf -- "perforate program\n\n"
    printf -- "-o, --output FILE   output file\n"
    printf -- "-a, --all           do all steps\n"
    printf -- "-e, --extract       extract scop\n"
    printf -- "-p, --perforate     perforate scop\n"
    printf -- "-g, --generate      generate perforated program\n"
}

extract_scop() {
    $CLAN "$1" > "$2"
}

perforate() {
    $PERF "$1" "$2" "$3" "$4"
}

run_pluto() {
    $PL
}

generate() {
    $CLOOG -openscop "$1" >"$2"
}

# default values
output_src=""
statement=0
level=0

while true; do
    case "$1" in
        -h|--help)
            usage
            exit
            ;;
        -a|--all)
            cmd_extract=yes
            cmd_perforate=yes
            cmd_generate=yes
            ;;
        -e|--extract)
            cmd_extract=yes
            ;;
        -p|--perforate)
            cmd_perforate=yes
            ;;
        -g|--generate)
            cmd_generate=yes
            ;;
        -o|--output)
            shift
            output_src="$1"
            ;;
        -s|--statement)
            shift
            statement="$1"
            ;;
        -l|--level)
            shift
            level="$1"
            ;;
        *)
            break
    esac
    shift
done

input_src=$1

scop="$input_src.openscop"
perforated="$input_src.perf.openscop"


if [ "$input_src" = "" ]; then
    printf "missing input\n"
    exit 1
fi

if [ "$cmd_extract" = "yes" ]; then
    extract_scop "$input_src" "$scop"
fi

if [ "$cmd_perforate" = "yes" ]; then
    perforate "$scop" "$perforated" "$statement" "$level"
fi

if [ "$cmd_generate" = "yes" ]; then
    generate "$perforated" "$output_src"
fi

printf "input is \"%s\" output is \"%s\"\n" "$input_src" "$output_src"
